@@ -36,13 +36,19 @@ from snimpy import mib
 
 PYTHON3 = sys.version_info >= (3, 0)
 if PYTHON3:
-    ord2 = lambda x: x
-    chr2 = lambda x: bytes([x & 0xff])
+    def ord2(x):
+        return x
+
+    def chr2(x):
+        return bytes([x & 0xff])
+
     unicode = str
     long = int
 else:
     ord2 = ord
-    chr2 = lambda x: chr(x & 0xff)
+
+    def chr2(x):
+        return chr(x & 0xff)
 
 
 def ordering_with_cmp(cls):
@@ -111,6 +117,12 @@ class Type(object):
             self._value = value
             self.entity = entity
 
+        if isinstance(self, String):
+            # Ensure that strings follow their format, if it is applied.
+            # This is safer and simpler than toOid, as it does not do
+            # additional validation.
+            self._toBytes()
+
         return self
 
     @classmethod
@@ -476,9 +488,9 @@ class String(StringOrOctetString, unicode):
                     fmatch = "(?P<d>[0-9]{{1,{0}}})".format(
                         int(length * 2.4083) + 1)
                 elif format == "a":
-                    fmatch = "(?P<a>.{{1,{0}}})".format(length)
+                    fmatch = "(?P<a>(?:.|\n){{1,{0}}})".format(length)
                 elif format == "t":
-                    fmatch = "(?P<t>.{{1,{0}}})".format(length)
+                    fmatch = "(?P<t>(?:.|\n){{1,{0}}})".format(length)
                 else:
                     raise ValueError("{0!r} cannot be parsed due to an "
                                      "incorrect format ({1})".format(
@@ -489,7 +501,7 @@ class String(StringOrOctetString, unicode):
                 if not mo:
                     raise ValueError("{0!r} cannot be parsed because it "
                                      "does not match format {1} at "
-                                     "index {i}".format(self._value, fmt, i))
+                                     "index {2}".format(self._value, fmt, i))
                 if format in ["o", "x", "d"]:
                     if format == "o":
                         r = int(mo.group("o"), 8)
