@@ -74,6 +74,8 @@ class Session(object):
     session. From such an instance, one can get information from the
     associated agent."""
 
+    _cmdgen = cmdgen.CommandGenerator()
+
     def __init__(self, host,
                  community="public", version=2,
                  secname=None,
@@ -114,7 +116,6 @@ class Session(object):
         """
         self._host = host
         self._version = version
-        self._cmdgen = cmdgen.CommandGenerator()
 
         # Put authentication stuff in self._auth
         if version in [1, 2]:
