@@ -1,4 +1,8 @@
-import unittest
+import sys
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
 import os
 import time
 from datetime import timedelta
@@ -137,6 +141,12 @@ class TestManagerGet(TestManager):
         self.assertRaises(ValueError,
                           self.manager.ifDescr.__getitem__, "nothing")
 
+    def testGetChangingStuff(self):
+        """Get stuff with varying values"""
+        initial = self.manager.ifInOctets[2]
+        current = self.manager.ifInOctets[2]
+        self.assertGreater(current, initial)
+
 
 class TestManagerSet(TestManager):
 
@@ -279,6 +289,26 @@ class TestCachingManager(TestManagerGet):
                                version=2, cache=1)
         self.session = self.manager._session._session
 
+    def testGetChangingStuff(self):
+        """Get stuff with varying values"""
+        initial = self.manager.ifInOctets[2]
+        current = self.manager.ifInOctets[2]
+        self.assertEqual(current, initial)
+
+    def testCacheFlush(self):
+        """Test cache timeout is working as expected"""
+        first1 = self.manager.ifInOctets[1]
+        second1 = self.manager.ifInOctets[2]
+        third1 = self.manager.ifInOctets[3]
+        time.sleep(0.5)
+        second2 = self.manager.ifInOctets[2]
+        third2 = self.manager.ifInOctets[3]
+        self.assertEqual(second1, second2)  # timeout not reached
+        self.assertEqual(third1, third2)  # timeout not reached
+        time.sleep(1)
+        first2 = self.manager.ifInOctets[1]
+        self.assertGreater(first2, first1)  # timeout was reached
+
 
 class TestCachingManagerWithModificatons(TestManager):
 
