@@ -1,7 +1,12 @@
 import unittest
 import os
+import sys
 from snimpy import mib, basictypes
 
+PYTHON3 = sys.version_info >= (3, 0)
+if PYTHON3:
+    unicode = str
+
 
 class TestMibSnimpy(unittest.TestCase):
 
@@ -13,13 +18,18 @@ class TestMibSnimpy(unittest.TestCase):
                       "snimpyTables"]
         self.nodes.sort()
         self.tables = ["snimpyComplexTable",
+                       "snimpyInetAddressTable",
                        "snimpySimpleTable",
-                       "snimpyIndexTable"]
+                       "snimpyIndexTable",
+                       "snimpyInvalidTable"]
         self.tables.sort()
         self.columns = ["snimpyComplexFirstIP",
                         "snimpyComplexSecondIP",
                         "snimpySimpleIndex",
                         "snimpyComplexState",
+                        "snimpyInetAddressType",
+                        "snimpyInetAddress",
+                        "snimpyInetAddressState",
                         "snimpySimpleDescr",
                         "snimpySimplePhys",
                         "snimpySimpleType",
@@ -29,6 +39,8 @@ class TestMibSnimpy(unittest.TestCase):
                         "snimpyIndexFixedLen",
                         "snimpyIndexImplied",
                         "snimpyIndexInt",
+                        "snimpyInvalidIndex",
+                        "snimpyInvalidDescr"
                         ]
         self.columns.sort()
         self.scalars = ["snimpyIpAddress",
@@ -45,9 +57,16 @@ class TestMibSnimpy(unittest.TestCase):
                         "snimpyNotImplemented",
                         "snimpyOctetString",
                         "snimpyUnicodeString",
-                        "snimpyMacAddress"]
+                        "snimpyMacAddress",
+                        "snimpyMacAddressInvalid"]
         self.scalars.sort()
 
+        self.expected_modules = [u"SNMPv2-SMI",
+                                 u"SNMPv2-TC",
+                                 u"SNIMPY-MIB",
+                                 u"INET-ADDRESS-MIB",
+                                 u"IANAifType-MIB"]
+
     def tearDown(self):
         mib.reset()
 
@@ -101,6 +120,37 @@ class TestMibSnimpy(unittest.TestCase):
             self.assertEqual(str(mib.get('SNIMPY-MIB', i)), i)
             self.assert_(isinstance(mib.get('SNIMPY-MIB', i), mib.Node))
 
+    def testGetByOid(self):
+        """Test that we can get all named attributes by OID."""
+        for i in self.scalars:
+            nodebyname = mib.get('SNIMPY-MIB', i)
+            self.assertEqual(str(mib.getByOid(nodebyname.oid)), i)
+            self.assert_(isinstance(mib.getByOid(nodebyname.oid), mib.Scalar))
+        for i in self.tables:
+            nodebyname = mib.get('SNIMPY-MIB', i)
+            self.assertEqual(str(mib.getByOid(nodebyname.oid)), i)
+            self.assert_(isinstance(mib.getByOid(nodebyname.oid), mib.Table))
+        for i in self.columns:
+            nodebyname = mib.get('SNIMPY-MIB', i)
+            self.assertEqual(str(mib.getByOid(nodebyname.oid)), i)
+            self.assert_(isinstance(mib.getByOid(nodebyname.oid), mib.Column))
+        for i in self.nodes:
+            nodebyname = mib.get('SNIMPY-MIB', i)
+            self.assertEqual(str(mib.getByOid(nodebyname.oid)), i)
+            self.assert_(isinstance(mib.getByOid(nodebyname.oid), mib.Node))
+
+    def testGetByOid_UnknownOid(self):
+        """Test that unknown OIDs raise an exception."""
+        self.assertRaises(mib.SMIException, mib.getByOid, (255,))
+
+    def testGetType(self):
+        """Test that _getType properly identifies known and unknown types."""
+        self.assertEqual(b"PhysAddress",
+                         mib.ffi.string(mib._getType("PhysAddress").name))
+        self.assertEqual(b"InetAddress",
+                         mib.ffi.string(mib._getType(b"InetAddress").name))
+        self.assertEqual(None, mib._getType("SomeUnknownType.kjgf"))
+
     def testTableColumnRelation(self):
         """Test that we can get the column from the table and vice-versa"""
         for i in self.tables:
@@ -189,6 +239,10 @@ class TestMibSnimpy(unittest.TestCase):
             [str(i)
              for i in mib.get("SNIMPY-MIB", "snimpyComplexTable").index],
             ["snimpyComplexFirstIP", "snimpyComplexSecondIP"])
+        self.assertEqual(
+            [str(i)
+             for i in mib.get("SNIMPY-MIB", "snimpyInetAddressTable").index],
+            ["snimpyInetAddressType", "snimpyInetAddress"])
 
     def testImplied(self):
         """Check that we can get implied attribute for a given table"""
@@ -221,6 +275,11 @@ class TestMibSnimpy(unittest.TestCase):
         for o in oids:
             self.assertEqual(mib.get('SNIMPY-MIB', o).oid, oids[o])
 
+    def testLoadedMibNames(self):
+        """Check that only expected modules were loaded."""
+        for module in self.expected_modules:
+            self.assertTrue(module in list(mib.loadedMibNames()))
+
     def testLoadInexistantModule(self):
         """Check that we get an exception when loading an inexistant module"""
         self.assertRaises(mib.SMIException, mib.load, "idontexist.gfdgfdg")
@@ -245,3 +304,111 @@ class TestMibSnimpy(unittest.TestCase):
         """Check that we get FMT from types"""
         self.assertEqual(mib.get("SNIMPY-MIB", 'snimpySimplePhys').fmt, "1x:")
         self.assertEqual(mib.get("SNIMPY-MIB", 'snimpyInteger').fmt, "d-2")
+
+    def testTypeOverrides(self):
+        """Check that we can override a type"""
+        table = mib.get("SNIMPY-MIB", "snimpyInetAddressTable")
+        addrtype_attr = table.index[0]
+        addr_attr = table.index[1]
+
+        # Try overriding to IPv4 with a byte string name.
+        addrtype = addrtype_attr.type(addrtype_attr, "ipv4")
+        self.assertEqual(addrtype, "ipv4")
+        addr_attr.typeName = b"InetAddressIPv4"
+        ipv4 = u"127.0.0.1"
+        ipv4_oid = (4, 127, 0, 0, 1)
+
+        addr = addr_attr.type(addr_attr, ipv4)
+        self.assertEqual(str(addr), ipv4)
+        self.assertEqual(addr.toOid(), ipv4_oid)
+
+        addr_len, addr = addr_attr.type.fromOid(addr_attr, ipv4_oid)
+        self.assertEqual(addr_len, ipv4_oid[0] + 1)
+        self.assertEqual(str(addr), ipv4)
+        self.assertEqual(addr.toOid(), ipv4_oid)
+
+        # Try both IPv6 and non-bytes name.
+        addrtype = addrtype_attr.type(addrtype_attr, "ipv6")
+        self.assertEqual(addrtype, "ipv6")
+        addr_attr.typeName = u"InetAddressIPv6"
+        # Snimpy does not use leading zeroes.
+        ipv6 = u'102:304:506:708:90a:b0c:d0e:f01'
+        ipv6_oid = (16, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
+                    0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x01)
+
+        addr = addr_attr.type(addr_attr, ipv6)
+        self.assertEqual(str(addr), ipv6)
+        self.assertEqual(addr.toOid(), ipv6_oid)
+
+        addr_len, addr = addr_attr.type.fromOid(addr_attr, ipv6_oid)
+        self.assertEqual(addr_len, ipv6_oid[0] + 1)
+        self.assertEqual(str(addr), ipv6)
+        self.assertEqual(addr.toOid(), ipv6_oid)
+
+        # Try a type from a different module (chosen because snmpwalk
+        # prints IPv6 addresses incorrectly).
+        ipv6_1xformat = u'1:2:3:4:5:6:7:8:9:a:b:c:d:e:f:1'
+        addr_attr.typeName = "PhysAddress"
+
+        addr = addr_attr.type(addr_attr, ipv6_1xformat)
+        self.assertEqual(str(addr), ipv6_1xformat)
+        self.assertEqual(addr.toOid(), ipv6_oid)
+
+        # Try overriding back to the default.
+        del addr_attr.typeName
+        addr_len, addr = addr_attr.type.fromOid(addr_attr, ipv4_oid)
+        self.assertEqual(bytes(addr), b"\x7f\0\0\1")
+
+    def testTypeOverrides_Errors(self):
+        table = mib.get("SNIMPY-MIB", "snimpyInetAddressTable")
+        attr = table.index[1]
+
+        # Value with the wrong type.
+        self.assertRaises(AttributeError, setattr, attr, "typeName", None)
+
+        # Unknown type.
+        self.assertRaises(mib.SMIException, setattr, attr, "typeName",
+                          "SomeUnknownType.kjgf")
+
+        # Incompatible basetype.
+        self.assertRaises(mib.SMIException, setattr, attr, "typeName",
+                          "InetAddressType")
+
+        # Parse error.
+        attr.typeName = "InetAddressIPv4"
+        self.assertRaises(ValueError, attr.type, attr, u"01:02:03:04")
+
+    def testTypeName(self):
+        """Check that we can get the current declared type name"""
+        table = mib.get("SNIMPY-MIB", "snimpyInetAddressTable")
+        attr = table.index[1]
+
+        self.assertEqual(attr.typeName, b"InetAddress")
+
+        attr.typeName = b"InetAddressIPv4"
+        self.assertEqual(attr.typeName, b"InetAddressIPv4")
+
+        attr.typeName = b"InetAddressIPv6"
+        self.assertEqual(attr.typeName, b"InetAddressIPv6")
+
+        attr = mib.get("SNIMPY-MIB", "snimpySimpleIndex")
+        self.assertEqual(attr.typeName, b"Integer32")
+
+
+class TestSmi(unittest.TestCase):
+
+    def testGetPath(self):
+        """Test we can get default SMI path"""
+        current = mib.path()
+        self.assertTrue(type(current), unicode)
+        self.assertNotEqual(mib.path(), u"")
+
+    def testSetPath(self):
+        """Test we can set path to some value"""
+        original = mib.path()
+        current = original + ":/some/other/directory"
+        try:
+            mib.path(current)
+            self.assertEqual(mib.path(), current)
+        finally:
+            mib.path(original)
